/**
 * Extract links such as URLs and email addresses from plain text (see {@link org.nibor.autolink.LinkExtractor}) and
 * process the text by rendering the links (see {@link org.nibor.autolink.Autolink}).
 */
package org.nibor.autolink;
